@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -80,7 +80,8 @@ namespace Ogre {
         Pass* pass;
         TextureUnitState* textureUnit;
         GpuProgramPtr program; // used when referencing a program, not when defining it
-        bool isProgramShadowCaster; // when referencing, are we in context of shadow caster
+        bool isVertexProgramShadowCaster; // when referencing, are we in context of shadow caster
+        bool isFragmentProgramShadowCaster; // when referencing, are we in context of shadow caster
         bool isVertexProgramShadowReceiver; // when referencing, are we in context of shadow caster
 		bool isFragmentProgramShadowReceiver; // when referencing, are we in context of shadow caster
         GpuProgramParametersSharedPtr programParams;
@@ -205,7 +206,7 @@ namespace Ogre {
         MaterialScriptContext mScriptContext;
 
         /** internal method for parsing a material
-        @returns true if it expects the next line to be a {
+        @return true if it expects the next line to be a {
         */
         bool parseScriptLine(String& line);
         /** internal method for finding & invoking an attribute parser. */
@@ -243,6 +244,7 @@ namespace Ogre {
         void writePass(const Pass* pPass);
         void writeVertexProgramRef(const Pass* pPass);
         void writeShadowCasterVertexProgramRef(const Pass* pPass);
+        void writeShadowCasterFragmentProgramRef(const Pass* pPass);
         void writeShadowReceiverVertexProgramRef(const Pass* pPass);
         void writeShadowReceiverFragmentProgramRef(const Pass* pPass);
         void writeFragmentProgramRef(const Pass* pPass);
@@ -417,6 +419,14 @@ namespace Ogre {
 			buffer += (" " + val);
 		}
 
+		String quoteWord(const String& val)
+		{
+			if (val.find_first_of(" \t") != String::npos)
+				return ("\"" + val + "\"");
+			else return val;
+		}
+
+
 		void writeComment(unsigned short level, const String& comment, const bool useMainBuffer = true)
 		{
             String& buffer = (useMainBuffer ? mBuffer : mGpuProgramBuffer);
