@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -96,6 +96,7 @@ namespace Ogre
         OverlayManager* mOverlayManager;
         FontManager* mFontManager;
         ArchiveFactory *mZipArchiveFactory;
+        ArchiveFactory *mEmbeddedZipArchiveFactory;
         ArchiveFactory *mFileSystemArchiveFactory;
 		ResourceGroupManager* mResourceGroupManager;
 		ResourceBackgroundQueue* mResourceBackgroundQueue;
@@ -113,6 +114,7 @@ namespace Ogre
         unsigned long mNextFrame;
 		Real mFrameSmoothingTime;
 		bool mRemoveQueueStructuresOnClear;
+		Real mDefaultMinPixelSize;
 
 	public:
 		typedef vector<DynLib*>::type PluginLibList;
@@ -142,6 +144,11 @@ namespace Ogre
 
 		WorkQueue* mWorkQueue;
 
+		///Tells whether blend indices information needs to be passed to the GPU
+		bool mIsBlendIndicesGpuRedundant;
+		///Tells whether blend weights information needs to be passed to the GPU
+		bool mIsBlendWeightsGpuRedundant;
+
         /** Method reads a plugins configuration file and instantiates all
             plugins.
             @param
@@ -221,7 +228,7 @@ namespace Ogre
                 from a previous run. If there is, the state of the system will
                 be restored to that configuration.
 
-            @returns
+            @return
                 If a valid configuration was found, <b>true</b> is returned.
             @par
                 If there is no saved configuration, or if the system failed
@@ -239,7 +246,7 @@ namespace Ogre
                 RenderSystem::setConfigOption and Root::saveConfig with the
                 user's choices. This is the easiest way to get the system
                 configured.
-            @returns
+            @return
                 If the user clicked 'Ok', <b>true</b> is returned.
             @par
                 If they clicked 'Cancel' (in which case the app should
@@ -271,7 +278,7 @@ namespace Ogre
         /** Retrieve a pointer to the render system by the given name
             @param
                 name Name of the render system intend to retrieve.
-            @returns
+            @return
                 A pointer to the render system, <b>NULL</b> if no found.
         */
         RenderSystem* getRenderSystemByName(const String& name);
@@ -310,7 +317,7 @@ namespace Ogre
                 Root::createRenderWindow). The window will be
                 created based on the options currently set on the render
                 system.
-            @returns
+            @return
                 A pointer to the automatically created window, if
                 requested, otherwise <b>NULL</b>.
         */
@@ -655,13 +662,19 @@ namespace Ogre
 		bool createRenderWindows(const RenderWindowDescriptionList& renderWindowDescriptions,
 			RenderWindowList& createdWindows);
 	
-        /** Detaches a RenderTarget from the active render system.
+        /** Detaches a RenderTarget from the active render system
+        and returns a pointer to it.
+        @note
+        If the render target cannot be found, NULL is returned.
         */
-        void detachRenderTarget( RenderTarget* pWin );
+        RenderTarget* detachRenderTarget( RenderTarget* pWin );
 
-        /** Detaches a named RenderTarget from the active render system.
+        /** Detaches a named RenderTarget from the active render system
+        and returns a pointer to it.
+        @note
+        If the render target cannot be found, NULL is returned.
         */
-        void detachRenderTarget( const String & name );
+        RenderTarget* detachRenderTarget( const String & name );
 
         /** Destroys the given RenderTarget.
         */
@@ -671,7 +684,7 @@ namespace Ogre
         */
         void destroyRenderTarget(const String &name);
 
-        /** Retrieves a pointer to the a named render window.
+        /** Retrieves a pointer to a named render target.
         */
         RenderTarget * getRenderTarget(const String &name);
 
@@ -742,7 +755,7 @@ namespace Ogre
             for you, then call the other version of this method with no parameters.
         @param evt Event object which includes all the timing information which you have 
             calculated for yourself
-        @returns False if one or more frame listeners elected that the rendering loop should
+        @return False if one or more frame listeners elected that the rendering loop should
             be terminated, true otherwise.
         */
         bool _fireFrameStarted(FrameEvent& evt);
@@ -770,7 +783,7 @@ namespace Ogre
             for you, then call the other version of this method with no parameters.
         @param evt Event object which includes all the timing information which you have 
             calculated for yourself
-        @returns False if one or more frame listeners elected that the rendering loop should
+        @return False if one or more frame listeners elected that the rendering loop should
             be terminated, true otherwise.
         */
         bool _fireFrameEnded(FrameEvent& evt);
@@ -788,7 +801,7 @@ namespace Ogre
             This method calculates the frame timing information for you based on the elapsed
             time. If you want to specify elapsed times yourself you should call the other 
             version of this method which takes event details as a parameter.
-        @returns False if one or more frame listeners elected that the rendering loop should
+        @return False if one or more frame listeners elected that the rendering loop should
             be terminated, true otherwise.
         */
         bool _fireFrameStarted();
@@ -814,7 +827,7 @@ namespace Ogre
             This method calculates the frame timing information for you based on the elapsed
             time. If you want to specify elapsed times yourself you should call the other 
             version of this method which takes event details as a parameter.
-        @returns False if one or more frame listeners elected that the rendering loop should
+        @return False if one or more frame listeners elected that the rendering loop should
             be terminated, true otherwise.
         */
         bool _fireFrameEnded();
@@ -854,7 +867,7 @@ namespace Ogre
             you may wish to call it to update all the render targets which are
             set to auto update (RenderTarget::setAutoUpdated). You can also update
             individual RenderTarget instances using their own update() method.
-		@returns false if a FrameListener indicated it wishes to exit the render loop
+		@return false if a FrameListener indicated it wishes to exit the render loop
         */
         bool _updateAllRenderTargets(void);
 
@@ -867,7 +880,7 @@ namespace Ogre
             you may wish to call it to update all the render targets which are
             set to auto update (RenderTarget::setAutoUpdated). You can also update
             individual RenderTarget instances using their own update() method.
-		@returns false if a FrameListener indicated it wishes to exit the render loop
+		@return false if a FrameListener indicated it wishes to exit the render loop
         */
         bool _updateAllRenderTargets(FrameEvent& evt);
 
@@ -1020,6 +1033,41 @@ namespace Ogre
 		*/
 		void setWorkQueue(WorkQueue* queue);
 			
+		/** Sets whether blend indices information needs to be passed to the GPU.
+			When entities use software animation they remove blend information such as
+			indices and weights from the vertex buffers sent to the graphic card. This function
+			can be used to limit which information is removed.
+		@param redundant Set to true to remove blend indices information.
+		*/
+		void setBlendIndicesGpuRedundant(bool redundant) {	mIsBlendIndicesGpuRedundant = redundant; }
+		/** Returns whether blend indices information needs to be passed to the GPU
+		see setBlendIndicesGpuRedundant() for more information
+		*/
+		bool isBlendIndicesGpuRedundant() const { return mIsBlendIndicesGpuRedundant; }
+
+		/** Sets whether blend weights information needs to be passed to the GPU.
+		When entities use software animation they remove blend information such as
+		indices and weights from the vertex buffers sent to the graphic card. This function
+		can be used to limit which information is removed.
+		@param redundant Set to true to remove blend weights information.
+		*/
+		void setBlendWeightsGpuRedundant(bool redundant) {	mIsBlendWeightsGpuRedundant = redundant; }
+		/** Returns whether blend weights information needs to be passed to the GPU
+		see setBlendWeightsGpuRedundant() for more information
+		*/
+		bool isBlendWeightsGpuRedundant() const { return mIsBlendWeightsGpuRedundant; }
+	
+		/** Set the default minimum pixel size for object to be rendered by
+		@note
+			To use this feature see Camera::setUseMinPixelSize()
+		*/
+		void setDefaultMinPixelSize(Real pixelSize) { mDefaultMinPixelSize = pixelSize; }
+
+		/** Get the default minimum pixel size for object to be rendered by
+		*/
+		Real getDefaultMinPixelSize() { return mDefaultMinPixelSize; }
+	
+
     };
 	/** @} */
 	/** @} */
