@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,7 @@ THE SOFTWARE.
 #include "OgreStableHeaders.h"
 
 #include "OgrePass.h"
+#include "OgreRoot.h"
 #include "OgreTechnique.h"
 #include "OgreException.h"
 #include "OgreGpuProgramUsage.h"
@@ -169,6 +170,7 @@ namespace Ogre {
 		, mLightsPerIteration(1)
 		, mRunOnlyForOneLightType(false)
 		, mOnlyLightType(Light::LT_POINT)
+		, mLightMask(0xFFFFFFFF)
 		, mShadeOptions(SO_GOURAUD)
 		, mPolygonMode(PM_SOLID)
 		, mNormaliseNormals(false)
@@ -181,6 +183,7 @@ namespace Ogre {
 		, mFogDensity(0.001)
 		, mVertexProgramUsage(0)
 		, mShadowCasterVertexProgramUsage(0)
+        , mShadowCasterFragmentProgramUsage(0)
 		, mShadowReceiverVertexProgramUsage(0)
 		, mFragmentProgramUsage(0)
 		, mShadowReceiverFragmentProgramUsage(0)
@@ -195,7 +198,7 @@ namespace Ogre {
 		, mContentTypeLookupBuilt(false)
 		, mLightScissoring(false)
 		, mLightClipPlanes(false)
-		, mIlluminationStage(IS_UNKNOWN)		
+		, mIlluminationStage(IS_UNKNOWN)
     {
 		mPointAttenuationCoeffs[0] = 1.0f;
 		mPointAttenuationCoeffs[1] = mPointAttenuationCoeffs[2] = 0.0f;
@@ -210,7 +213,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
 	Pass::Pass(Technique *parent, unsigned short index, const Pass& oth)
         :mParent(parent), mIndex(index), mVertexProgramUsage(0), mShadowCasterVertexProgramUsage(0), 
-		mShadowReceiverVertexProgramUsage(0), mFragmentProgramUsage(0), 
+		mShadowCasterFragmentProgramUsage(0), mShadowReceiverVertexProgramUsage(0), mFragmentProgramUsage(0), 
 		mShadowReceiverFragmentProgramUsage(0), mGeometryProgramUsage(0),
 		mQueuedForDeletion(false), mPassIterationCount(1)
     {
@@ -228,6 +231,7 @@ namespace Ogre {
 		OGRE_DELETE mVertexProgramUsage;
 		OGRE_DELETE mFragmentProgramUsage;
 		OGRE_DELETE mShadowCasterVertexProgramUsage;
+        OGRE_DELETE mShadowCasterFragmentProgramUsage;
 		OGRE_DELETE mShadowReceiverVertexProgramUsage;
 		OGRE_DELETE mShadowReceiverFragmentProgramUsage;		
     }
@@ -299,6 +303,7 @@ namespace Ogre {
 		mLightScissoring = oth.mLightScissoring;
 		mLightClipPlanes = oth.mLightClipPlanes;
 		mIlluminationStage = oth.mIlluminationStage;
+		mLightMask = oth.mLightMask;
 
 		OGRE_DELETE mVertexProgramUsage;
 		if (oth.mVertexProgramUsage)
@@ -320,6 +325,16 @@ namespace Ogre {
             mShadowCasterVertexProgramUsage = NULL;
         }
 
+		OGRE_DELETE mShadowCasterFragmentProgramUsage;
+        if (oth.mShadowCasterFragmentProgramUsage)
+        {
+            mShadowCasterFragmentProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mShadowCasterFragmentProgramUsage), this);
+        }
+        else
+        {
+            mShadowCasterFragmentProgramUsage = NULL;
+        }
+
 		OGRE_DELETE mShadowReceiverVertexProgramUsage;
         if (oth.mShadowReceiverVertexProgramUsage)
         {
@@ -1010,6 +1025,16 @@ namespace Ogre {
 		return mStartLight;
 	}
 	//-----------------------------------------------------------------------
+	void Pass::setLightMask(uint32 mask)
+	{
+		mLightMask = mask;
+	}
+	//-----------------------------------------------------------------------
+	uint32 Pass::getLightMask() const
+	{
+		return mLightMask;
+	}
+	//-----------------------------------------------------------------------
 	void Pass::setLightCountPerIteration(unsigned short c)
 	{
 		mLightsPerIteration = c;
@@ -1231,6 +1256,11 @@ namespace Ogre {
             // Load vertex program
             mShadowCasterVertexProgramUsage->_load();
         }
+        if (mShadowCasterFragmentProgramUsage)
+        {
+            // Load fragment program
+            mShadowCasterFragmentProgramUsage->_load();
+        }
         if (mShadowReceiverVertexProgramUsage)
         {
             // Load vertex program
@@ -1628,6 +1658,11 @@ namespace Ogre {
             OGRE_DELETE mShadowCasterVertexProgramUsage;
             mShadowCasterVertexProgramUsage = 0;
         }
+        if (mShadowCasterFragmentProgramUsage)
+        {
+            OGRE_DELETE mShadowCasterFragmentProgramUsage;
+            mShadowCasterFragmentProgramUsage = 0;
+        }
         if (mShadowReceiverVertexProgramUsage)
         {
             OGRE_DELETE mShadowReceiverVertexProgramUsage;
@@ -1727,6 +1762,67 @@ namespace Ogre {
         return mShadowCasterVertexProgramUsage->getProgram();
     }
     //-----------------------------------------------------------------------
+    void Pass::setShadowCasterFragmentProgram(const String& name)
+    {
+        // Turn off fragment program if name blank
+        if (name.empty())
+        {
+            if (mShadowCasterFragmentProgramUsage) OGRE_DELETE mShadowCasterFragmentProgramUsage;
+            mShadowCasterFragmentProgramUsage = NULL;
+        }
+        else
+        {
+            if (!mShadowCasterFragmentProgramUsage)
+            {
+                mShadowCasterFragmentProgramUsage = OGRE_NEW GpuProgramUsage(GPT_FRAGMENT_PROGRAM, this);
+            }
+            mShadowCasterFragmentProgramUsage->setProgramName(name);
+        }
+        // Needs recompilation
+        mParent->_notifyNeedsRecompile();
+    }
+    //-----------------------------------------------------------------------
+    void Pass::setShadowCasterFragmentProgramParameters(GpuProgramParametersSharedPtr params)
+    {
+        if (Ogre::Root::getSingletonPtr()->getRenderSystem()->getName().find("OpenGL ES 2") != String::npos)
+        {
+            if (!mShadowCasterFragmentProgramUsage)
+            {
+                OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
+                    "This pass does not have a shadow caster fragment program assigned!",
+                    "Pass::setShadowCasterFragmentProgramParameters");
+            }
+            mShadowCasterFragmentProgramUsage->setParameters(params);
+        }
+    }
+    //-----------------------------------------------------------------------
+    const String& Pass::getShadowCasterFragmentProgramName(void) const
+    {
+        if (!mShadowCasterFragmentProgramUsage)
+            return StringUtil::BLANK;
+        else
+            return mShadowCasterFragmentProgramUsage->getProgramName();
+    }
+    //-----------------------------------------------------------------------
+    GpuProgramParametersSharedPtr Pass::getShadowCasterFragmentProgramParameters(void) const
+    {
+        if (Ogre::Root::getSingletonPtr()->getRenderSystem()->getName().find("OpenGL ES 2") != String::npos)
+        {
+            if (!mShadowCasterFragmentProgramUsage)
+            {
+                OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
+                    "This pass does not have a shadow caster fragment program assigned!",
+                    "Pass::getShadowCasterFragmentProgramParameters");
+            }
+        }
+        return mShadowCasterFragmentProgramUsage->getParameters();
+    }
+    //-----------------------------------------------------------------------
+    const GpuProgramPtr& Pass::getShadowCasterFragmentProgram(void) const
+    {
+        return mShadowCasterFragmentProgramUsage->getProgram();
+    }
+    //-----------------------------------------------------------------------
     void Pass::setShadowReceiverVertexProgram(const String& name)
     {
         // Turn off vertex program if name blank
