@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,15 +35,18 @@ THE SOFTWARE.
 #include "OgreRenderTargetListener.h"
 #include "OgreRoot.h"
 #include "OgreRenderSystem.h"
+#include "OgreDepthBuffer.h"
 
 namespace Ogre {
 
     RenderTarget::RenderTarget()
-		:mPriority(OGRE_DEFAULT_RT_GROUP),
-		mActive(true),
-		mAutoUpdate(true),
-		mHwGamma(false), 
-		mFSAA(0)
+		: mPriority(OGRE_DEFAULT_RT_GROUP)
+		, mDepthBufferPoolId(DepthBuffer::POOL_DEFAULT)
+		, mDepthBuffer(0)
+		, mActive(true)
+		, mAutoUpdate(true)
+		, mHwGamma(false)
+		, mFSAA(0)
     {
         mTimer = Root::getSingleton().getTimer();
         resetStatistics();
@@ -59,6 +62,9 @@ namespace Ogre {
             OGRE_DELETE (*i).second;
         }
 
+		//DepthBuffer keeps track of us, avoid a dangling pointer
+		detachDepthBuffer();
+
 
         // Write closing message
 		LogManager::getSingleton().stream(LML_TRIVIAL)
@@ -94,6 +100,53 @@ namespace Ogre {
     {
         return mColourDepth;
     }
+	//-----------------------------------------------------------------------
+	void RenderTarget::setDepthBufferPool( uint16 poolId )
+	{
+		if( mDepthBufferPoolId != poolId )
+		{
+			mDepthBufferPoolId = poolId;
+			detachDepthBuffer();
+		}
+	}
+	//-----------------------------------------------------------------------
+	uint16 RenderTarget::getDepthBufferPool() const
+	{
+		return mDepthBufferPoolId;
+	}
+	//-----------------------------------------------------------------------
+	DepthBuffer* RenderTarget::getDepthBuffer() const
+	{
+		return mDepthBuffer;
+	}
+	//-----------------------------------------------------------------------
+	bool RenderTarget::attachDepthBuffer( DepthBuffer *depthBuffer )
+	{
+		bool retVal = false;
+
+		if( (retVal = depthBuffer->isCompatible( this )) )
+		{
+			detachDepthBuffer();
+			mDepthBuffer = depthBuffer;
+			mDepthBuffer->_notifyRenderTargetAttached( this );
+		}
+
+		return retVal;
+	}
+	//-----------------------------------------------------------------------
+	void RenderTarget::detachDepthBuffer()
+	{
+		if( mDepthBuffer )
+		{
+			mDepthBuffer->_notifyRenderTargetDetached( this );
+			mDepthBuffer = 0;
+		}
+	}
+	//-----------------------------------------------------------------------
+	void RenderTarget::_detachDepthBuffer()
+	{
+		mDepthBuffer = 0;
+	}
 
     void RenderTarget::updateImpl(void)
     {
@@ -168,7 +221,7 @@ namespace Ogre {
 
     Viewport* RenderTarget::addViewport(Camera* cam, int ZOrder, float left, float top ,
         float width , float height)
-    {
+    {		
         // Check no existing viewport with this Z-order
         ViewportList::iterator it = mViewportList.find(ZOrder);
 
@@ -394,6 +447,23 @@ namespace Ogre {
         while (index--)
             ++i;
         return i->second;
+    }
+	//-----------------------------------------------------------------------
+    Viewport* RenderTarget::getViewportByZOrder(int ZOrder)
+    {
+		ViewportList::iterator i = mViewportList.find(ZOrder);
+		if(i == mViewportList.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,"No viewport with given zorder : "
+				+ StringConverter::toString(ZOrder), "RenderTarget::getViewportByZOrder");
+		}
+        return i->second;
+    }
+	//-----------------------------------------------------------------------
+    bool RenderTarget::hasViewportWithZOrder(int ZOrder)
+    {
+		ViewportList::iterator i = mViewportList.find(ZOrder);
+		return i != mViewportList.end();
     }
     //-----------------------------------------------------------------------
     bool RenderTarget::isActive() const
