@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -489,7 +489,7 @@ namespace Ogre {
 	void DefaultWorkQueueBase::processResponses() 
 	{
 		unsigned long msStart = Root::getSingleton().getTimer()->getMilliseconds();
-		unsigned long msCurrent = msStart;
+		unsigned long msCurrent = 0;
 
 		// keep going until we run out of responses or out of time
 		while(true)
