@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -58,26 +58,72 @@ void OSXGLSupport::addConfig( void )
 	ConfigOption optFullScreen;
 	ConfigOption optVideoMode;
 	ConfigOption optBitDepth;
-    ConfigOption optFSAA;
+	ConfigOption optFSAA;
 	ConfigOption optRTTMode;
     ConfigOption optMacAPI;
+	ConfigOption optHiddenWindow;
+	ConfigOption optVsync;
+	ConfigOption optSRGB;
+#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+	ConfigOption optEnableFixedPipeline;
+#endif
 
-	// FS setting possiblities
+	// FS setting possibilities
 	optFullScreen.name = "Full Screen";
 	optFullScreen.possibleValues.push_back( "Yes" );
 	optFullScreen.possibleValues.push_back( "No" );
 	optFullScreen.currentValue = "No";
 	optFullScreen.immutable = false;
 
+    // Hidden window setting possibilities
+	optHiddenWindow.name = "hidden";
+	optHiddenWindow.possibleValues.push_back( "Yes" );
+	optHiddenWindow.possibleValues.push_back( "No" );
+	optHiddenWindow.currentValue = "No";
+	optHiddenWindow.immutable = false;
+
+    // FS setting possibilities
+	optVsync.name = "vsync";
+	optVsync.possibleValues.push_back( "Yes" );
+	optVsync.possibleValues.push_back( "No" );
+	optVsync.currentValue = "No";
+	optVsync.immutable = false;
+
 	optBitDepth.name = "Colour Depth";
 	optBitDepth.possibleValues.push_back( "32" );
 	optBitDepth.possibleValues.push_back( "16" );
 	optBitDepth.currentValue = "32";
 	optBitDepth.immutable = false;
 
+    optRTTMode.name = "RTT Preferred Mode";
+	optRTTMode.possibleValues.push_back( "FBO" );
+	optRTTMode.possibleValues.push_back( "PBuffer" );
+	optRTTMode.possibleValues.push_back( "Copy" );
+	optRTTMode.currentValue = "FBO";
+	optRTTMode.immutable = false;
+
+	// SRGB on auto window
+	optSRGB.name = "sRGB Gamma Conversion";
+	optSRGB.possibleValues.push_back("Yes");
+	optSRGB.possibleValues.push_back("No");
+	optSRGB.currentValue = "No";
+	optSRGB.immutable = false;
+
+#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+		optEnableFixedPipeline.name = "Fixed Pipeline Enabled";
+		optEnableFixedPipeline.possibleValues.push_back( "Yes" );
+		optEnableFixedPipeline.possibleValues.push_back( "No" );
+		optEnableFixedPipeline.currentValue = "Yes";
+		optEnableFixedPipeline.immutable = false;
+#endif
+
     mOptions[ optFullScreen.name ] = optFullScreen;
 	mOptions[ optBitDepth.name ] = optBitDepth;
 
+#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+		mOptions[optEnableFixedPipeline.name] = optEnableFixedPipeline;
+#endif
+
 	CGLRendererInfoObj rend;
 
 #if defined(MAC_OS_X_VERSION_10_4) && MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
@@ -89,10 +135,10 @@ void OSXGLSupport::addConfig( void )
 #endif
 
 #if defined(MAC_OS_X_VERSION_10_4) && MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-	long maxSamples;
+	long maxSamples = 0;
 	CGLDescribeRenderer(rend, 0, kCGLRPMaxSamples, &maxSamples);
 #else
-    GLint maxSamples;
+    GLint maxSamples = 0;
 	CGLDescribeRenderer(rend, 0, kCGLRPMaxSamples, &maxSamples);
 #endif
 
@@ -100,40 +146,19 @@ void OSXGLSupport::addConfig( void )
 
     // FSAA possibilities
     optFSAA.name = "FSAA";
-    optFSAA.possibleValues.push_back( "0" );
 
-	switch( maxSamples )
-	{
-		case 8:
-            optFSAA.possibleValues.push_back( "2" );
-            optFSAA.possibleValues.push_back( "4" );
-            optFSAA.possibleValues.push_back( "6" );
-            optFSAA.possibleValues.push_back( "8" );
-            break;
-		case 6:
-			optFSAA.possibleValues.push_back( "2" );
-			optFSAA.possibleValues.push_back( "4" );
-			optFSAA.possibleValues.push_back( "6" );
-			break;
-		case 4:
-			optFSAA.possibleValues.push_back( "2" );
-			optFSAA.possibleValues.push_back( "4" );
-			break;
-		case 2:
-			optFSAA.possibleValues.push_back( "2" );
-			break;
-		default: break;
-	}
+    for(int i = 0; i <= maxSamples; i += 2)
+        optFSAA.possibleValues.push_back( StringConverter::toString(i) );
 
     optFSAA.currentValue = "0";
     optFSAA.immutable = false;
 
     mOptions[ optFSAA.name ] = optFSAA;
 
-	// Video mode possiblities
+	// Video mode possibilities
 	optVideoMode.name = "Video Mode";
 	optVideoMode.immutable = false;
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
 	CFArrayRef displayModes = CGDisplayCopyAllDisplayModes(CGMainDisplayID(), NULL);
 #else
 	CFArrayRef displayModes = CGDisplayAvailableModes(CGMainDisplayID());
@@ -145,7 +170,7 @@ void OSXGLSupport::addConfig( void )
 	// Grab all the available display modes, then weed out duplicates...
 	for(int i = 0; i < numModes; ++i)
 	{
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
 		CGDisplayModeRef modeInfo = (CGDisplayModeRef)CFArrayGetValueAtIndex(displayModes, i);
 
         // Get IOKit flags for the display mode
@@ -205,7 +230,12 @@ void OSXGLSupport::addConfig( void )
 #endif
 		}
 	}
-	
+
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
+        // Release memory
+        CFRelease(displayModes);
+#endif
+
 	// Sort the modes...
 	CFArraySortValues(goodModes, CFRangeMake(0, CFArrayGetCount(goodModes)), 
 					  (CFComparatorFunction)_compareModes, NULL);
@@ -213,7 +243,7 @@ void OSXGLSupport::addConfig( void )
 	// Now pull the modes out and put them into optVideoModes
 	for(int i = 0; i < CFArrayGetCount(goodModes); ++i)
 	{
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
 		CGDisplayModeRef resolution = (CGDisplayModeRef)CFArrayGetValueAtIndex(goodModes, i);
 		
 		size_t fWidth  = CGDisplayModeGetWidth(resolution);
@@ -249,10 +279,13 @@ void OSXGLSupport::addConfig( void )
     optMacAPI.immutable = false;
 
     mOptions[optMacAPI.name] = optMacAPI;
-    mOptions[optFullScreen.name] = optFullScreen;
+	mOptions[optFullScreen.name] = optFullScreen;
 	mOptions[optVideoMode.name] = optVideoMode;
     mOptions[optFSAA.name] = optFSAA;
 	mOptions[optRTTMode.name] = optRTTMode;
+	mOptions[optHiddenWindow.name] = optHiddenWindow;
+	mOptions[optVsync.name] = optVsync;
+	mOptions[optSRGB.name] = optSRGB;
 }
 
 String OSXGLSupport::validateConfig( void )
@@ -287,6 +320,31 @@ RenderWindow* OSXGLSupport::createWindow( bool autoCreateWindow, GLRenderSystem*
         {
 			winOptions[ "FSAA" ] = opt->second.currentValue;
         }
+
+        opt = mOptions.find( "hidden" );
+        if( opt != mOptions.end() )
+        {
+            winOptions[ "hidden" ] = opt->second.currentValue;
+        }
+
+        opt = mOptions.find( "vsync" );
+        if( opt != mOptions.end() )
+        {
+            winOptions[ "vsync" ] = opt->second.currentValue;
+        }
+
+        opt = mOptions.find( "sRGB Gamma Conversion" );
+        if( opt != mOptions.end() )
+            winOptions["gamma"] = opt->second.currentValue;
+
+#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+			opt = mOptions.find("Fixed Pipeline Enabled");
+			if (opt == mOptions.end())
+				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find Fixed Pipeline enabled options!", "Win32GLSupport::createWindow");
+			bool enableFixedPipeline = (opt->second.currentValue == "Yes");
+			renderSystem->setFixedPipelineEnabled(enableFixedPipeline);
+#endif
+
         opt = mOptions.find( "macAPI" );
         if( opt != mOptions.end() )
         {
@@ -311,7 +369,7 @@ RenderWindow* OSXGLSupport::newWindow( const String &name, unsigned int width, u
 	
 	if(miscParams)
 	{
-        NameValuePairList::const_iterator opt(NULL);
+        NameValuePairList::const_iterator opt;
 
         // First we must determine if this is a Carbon or a Cocoa window
         // that we wish to create
@@ -399,7 +457,7 @@ GLPBuffer* OSXGLSupport::createPBuffer(PixelComponentType format, size_t width,
 		return OGRE_NEW OSXPBuffer(format, width, height);
 }
 
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
 CFComparisonResult OSXGLSupport::_compareModes (const void *val1, const void *val2, void *context)
 {
 	// These are the values we will be interested in...
