@@ -5,7 +5,7 @@ This source file is part of OGRE
 For the latest info, see http://www.ogre3d.org/
 
 Copyright (c) 2008 Renato Araujo Oliveira Filho <renatox@gmail.com>
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2012 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -69,7 +69,7 @@ namespace Ogre {
     GLESHardwarePixelBuffer::~GLESHardwarePixelBuffer()
     {
         // Force free buffer
-        OGRE_DELETE [] (uint8*)mBuffer.data;
+        delete [] (uint8*)mBuffer.data;
     }
 
     void GLESHardwarePixelBuffer::allocateBuffer()
@@ -78,7 +78,7 @@ namespace Ogre {
             // Already allocated
             return;
 
-        mBuffer.data = OGRE_NEW_FIX_FOR_WIN32 uint8[mSizeInBytes];
+        mBuffer.data = new uint8[mSizeInBytes];
         // TODO use PBO if we're HBU_DYNAMIC
     }
 
@@ -87,7 +87,7 @@ namespace Ogre {
         // Free buffer if we're STATIC to save memory
         if (mUsage & HBU_STATIC)
         {
-            OGRE_DELETE [] (uint8*)mBuffer.data;
+            delete [] (uint8*)mBuffer.data;
             mBuffer.data = 0;
         }
     }
@@ -131,8 +131,7 @@ namespace Ogre {
             src.getHeight() != dstBox.getHeight() ||
             src.getDepth() != dstBox.getDepth())
         {
-            // Scale to destination size. Use DevIL and not iluScale because ILU screws up for 
-            // floating point textures and cannot cope with 3D images.
+            // Scale to destination size.
             // This also does pixel format conversion if needed
             allocateBuffer();
             scaled = mBuffer.getSubVolume(dstBox);
@@ -290,10 +289,10 @@ namespace Ogre {
             {
                 String name;
                 name = "rtt/" + StringConverter::toString((size_t)this) + "/" + baseName;
-                GLESSurfaceDesc target;
-                target.buffer = this;
-                target.zoffset = zoffset;
-                RenderTexture *trt = GLESRTTManager::getSingleton().createRenderTexture(name, target, writeGamma, fsaa);
+                GLESSurfaceDesc surface;
+                surface.buffer = this;
+                surface.zoffset = zoffset;
+                RenderTexture *trt = GLESRTTManager::getSingleton().createRenderTexture(name, surface, writeGamma, fsaa);
                 mSliceTRT.push_back(trt);
                 Root::getSingleton().getRenderSystem()->attachRenderTarget(*mSliceTRT[zoffset]);
             }
@@ -748,8 +747,6 @@ namespace Ogre {
         scaled.front = data.front;
         scaled.back = data.back;
 
-        GLenum glFormat = GLESPixelUtil::getGLOriginFormat(data.format);
-        GLenum dataType = GLESPixelUtil::getGLOriginDataType(data.format);
         width = data.getWidth();
         height = data.getHeight();
 
@@ -759,8 +756,8 @@ namespace Ogre {
 
         for (int mip = 0; mip <= level; mip++)
         {
-            glFormat = GLESPixelUtil::getGLOriginFormat(scaled.format);
-            dataType = GLESPixelUtil::getGLOriginDataType(scaled.format);
+            GLenum glFormat = GLESPixelUtil::getGLOriginFormat(scaled.format);
+            GLenum dataType = GLESPixelUtil::getGLOriginDataType(scaled.format);
 
             glTexImage2D(GL_TEXTURE_2D,
                          mip,
@@ -775,7 +772,7 @@ namespace Ogre {
 
             if (mip != 0)
             {
-                OGRE_DELETE[] (uint8*) scaled.data;
+                delete[] (uint8*) scaled.data;
                 scaled.data = 0;
             }
 
@@ -792,7 +789,7 @@ namespace Ogre {
             int sizeInBytes = PixelUtil::getMemorySize(width, height, 1,
                                                        data.format);
             scaled = PixelBox(width, height, 1, data.format);
-            scaled.data = OGRE_NEW_FIX_FOR_WIN32 uint8[sizeInBytes];
+            scaled.data = new uint8[sizeInBytes];
             Image::scale(data, scaled, Image::FILTER_LINEAR);
         }
     }
@@ -833,4 +830,4 @@ namespace Ogre {
                                      GL_RENDERBUFFER_OES, mRenderbufferID);
         GL_CHECK_ERROR;
     }
-};
+}
